const { app, BrowserWindow } = require('electron');
const file = require('./helper/file');
const window = require('./helper/window');
const path = require('path');
const { execSync } = require('child_process');
const { program } = require('commander');

app.allowRendererProcessReuse = true;
// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) { // eslint-disable-line global-require
  app.quit();
}

/* --- Initialisation --- */

// Global var
let mainWindow;
let launchInfo;
let indexFile = 'src/index.html';

// Add version and option
program.version('0.0.1');
program
  .option('-s, --src', 'Load the file from src folder instead of the app folder');
program.parse(process.argv);

// Check Option
const folder = program.src ? '/' : '/app/';

/* --- Functions --- */

// Main function
async function main() {
  // Check if launch-info.json exist
  console.log('Reading launch info');
  if (file.exist(path.join(__dirname, '/data/launch-info.json'))) {
    launchInfo = require('./data/launch-info');
    // Reset data if change source between src and app
    if (launchInfo.app === program.src) {
      indexFile = 'src/reset.html';
    }
    // Update launch info
    launchInfo.app = !program.src;
  } else {
    launchInfo = {
      app: !program.src
    }
  }
  // Edit launch info file
  file.put(path.join(__dirname, '/data/launch-info.json'), JSON.stringify(launchInfo));
  // if launch the advanced electron app
  if (!program.src) {
    // Loading main process files
    loadMainProcessFiles();
    // Compile SCSS file from app
    console.info('Compiling SCSS files in CSS');
    let scssFile = path.join(__dirname, folder, 'src/css/style.scss');
    let cssFile = path.join(__dirname, folder, 'src/css/style.min.css');
    execSync(`node-sass --output-style compressed ${scssFile} ${cssFile}`);
    scssFile = path.join(__dirname, folder, 'src/css/materialize/materialize.scss');
    cssFile = path.join(__dirname, folder, 'src/css/materialize.min.css');
    execSync(`node-sass --output-style compressed ${scssFile} ${cssFile}`);
  }
  // Load main window
  console.info('Load main window');
  createMainWindow(program.src);
}

// Create main window function
function createMainWindow(simple = false) {
  if (simple) {
    mainWindow = window.simple(path.join(__dirname, folder, indexFile));
  } else {
    mainWindow = window.new(path.join(__dirname, folder, indexFile));
  }
  mainWindow.on('close', () => {
    mainWindow = null;
  });
}

function loadMainProcessFiles() {
  const mainProcessFiles = file.list(path.join(__dirname, folder, 'main/'));
  mainProcessFiles.forEach(f => {
    require(path.join(__dirname, folder, 'main/', f));
  });
}

/* --- Electron app actions --- */

// This method will be called when Electron is ready
app.on('ready', main);
// Quit when all windows are closed.
app.on('window-all-closed', () => {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit();
  }
});
// Re-open
app.on('activate', () => {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (BrowserWindow.getAllWindows().length === 0) {
    createMainWindow(program.src);
  }
});
 
